@@ -1439,8 +1439,10 @@ ps_truetype_error:
 	if (b.buf)
 		gnome_print_buffer_munmap (&b);
 
-	if (subfont_file)
+	if (subfont_file) {
 		unlink (subfont_file);
+		g_free (subfont_file);
+	}
 }	
 
 /**
