@@ -28,7 +28,9 @@
 #include <math.h>
 #include <string.h>
 #include <time.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <png.h>
 
 #include <libart_lgpl/art_affine.h>
@@ -727,8 +729,6 @@ static void _png_flush(png_structp png_ptr)
 {
 }
 
-#define EPSILON 1e-9
-
 static gint gnome_print_svg_image (GnomePrintContext *pc, const gdouble *a, 
 				   const guchar *px, gint w, gint h, gint rowstride, gint ch)
 {
@@ -767,7 +767,7 @@ static gint gnome_print_svg_image (GnomePrintContext *pc, const gdouble *a,
 		
 		g_warning("PNG encoding problems\n");
 
-		return GNOME_PRINT_OK;
+		return GNOME_PRINT_ERROR_UNKNOWN;
 	}
 
 	pBB = g_byte_array_new();
@@ -793,9 +793,7 @@ static gint gnome_print_svg_image (GnomePrintContext *pc, const gdouble *a,
 		png_write_rows(png_ptr, &pRow, 1);
 	}
 
-	/*
-	  Wrap things up with libpng
-	*/
+	/* Wrap things up with libpng */
 	png_write_end(png_ptr, info_ptr);
 
 	/* clean up after the write, and free any memory allocated */
@@ -878,6 +876,8 @@ static gunichar gnome_print_unichar_for_glyph (GnomePrintSvg * svg, GnomeFont *
 
 #undef UCS2_UPPER_BOUND
 
+#define EPSILON 1e-9
+
 static gint gnome_print_svg_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphList *gl)
 {
 	GnomePrintSvg * svg;
