@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  gnome-print-cups.c: A cups backend thingy
  *
@@ -44,9 +44,10 @@
 
 #include "libgnomeprint/gnome-print-i18n.h"
 
+static 	GModule *handle = NULL;
 /* Argument order: id, name */
 
-xmlChar *model_xml_template = 
+static const xmlChar *model_xml_template =
 "<?xml version=\"1.0\"?>"
 "<Model Id=\"%s\" Version=\"1.0\">"
 "  <Name>%s</Name>"
@@ -93,7 +94,7 @@ xmlChar *model_xml_template =
 
 /* Argument order: id */
 
-xmlChar *model_unknown_xml_template = 
+static const xmlChar *model_unknown_xml_template =
 "<?xml version=\"1.0\"?>"
 "<Model Id=\"%s\" Version=\"1.0\">"
 "  <Name>Unavailable PPD File</Name>"
@@ -141,75 +142,140 @@ xmlChar *model_unknown_xml_template =
 "  </Options>"
 "</Model>";
 
-struct GnomePrintCupsNewPrinterCbData
-{
+typedef struct {
 	GPAList *list;
-	char *path;
-};
+	char *module_path;
+} GnomePrintCupsNewPrinterCbData;
+
+typedef struct {
+	GnomeCupsPrinter *printer;
+	GPAList *list;
+	char *module_path;
+} GnomePrintCupsLoadPrinterData;
 
 G_MODULE_EXPORT void gpa_module_load_data (GPAPrinter *printer);
-static void printer_added_cb (const char *name, struct GnomePrintCupsNewPrinterCbData *data);
-static void printer_gone_cb (GnomeCupsPrinter *printer, GPAList *list);
+
+static void cb_get_ppd (guint id, ppd_file_t *ppd, GError **error, gpointer user_data);
+
+static void
+free_ppd_retrieval_data (gpointer p)
+{
+ 	GnomePrintCupsLoadPrinterData *data = p;
+ 	g_object_unref (data->printer);
+ 	g_free (data->module_path);
+ 	g_free (data);
+}
+
+static void
+start_printer_ppd_retrieval (GnomeCupsPrinter *printer,
+ 			     GPAList *printers_list,
+ 			     const char *module_path)
+{
+ 	GnomePrintCupsLoadPrinterData *loaddata;
+ 	loaddata = g_new0 (GnomePrintCupsLoadPrinterData, 1);
+ 	loaddata->printer = g_object_ref (printer);
+ 	loaddata->list = printers_list;
+ 	loaddata->module_path = g_strdup (module_path);
+ 	gnome_cups_printer_get_ppd_async (printer,
+		cb_get_ppd, loaddata, free_ppd_retrieval_data);
+}
+
+static void
+cb_printer_attributes_changed (GnomeCupsPrinter *printer,
+			       GnomePrintCupsNewPrinterCbData *data)
+{
+	start_printer_ppd_retrieval (printer, data->list, data->module_path);
+	g_object_unref (printer);
+}
+
+static void
+cb_printer_added (const char *name, GnomePrintCupsNewPrinterCbData *data)
+{
+	GPANode *node;
+	GnomeCupsPrinter *printer;
+
+	if ((node = gpa_printer_get_by_id (name)) != NULL) {
+		gpa_node_unref (node);
+		return;
+	}
+
+	printer = gnome_cups_printer_get (name);
+	if (!gnome_cups_printer_get_attributes_initialized (printer))
+		g_signal_connect (printer, "attributes-changed",
+			G_CALLBACK (cb_printer_attributes_changed), data);
+	else
+		cb_printer_attributes_changed (printer, data);
+}
+
+static void
+cb_printer_removed (const char *name, GPAList *list)
+{
+	GPANode *child = gpa_node_get_child (GPA_NODE (list), NULL);
+	for (; NULL != child ; child = gpa_node_get_child (GPA_NODE (list), child))
+		if (GPA_NODE_ID_COMPARE (child, name))
+			break;
+	if (NULL != child)
+		gpa_node_detach (child);
+}
+
+static void
+warn_of_unknown_encoding (char const *encoding)
+{
+	static GHashTable *unknown_encodings = NULL;
+
+	if (encoding == NULL)
+		return;
+
+	if (unknown_encodings == NULL)
+		unknown_encodings = g_hash_table_new_full (
+			g_str_hash, g_str_equal, g_free, NULL);
+	else if (NULL != g_hash_table_lookup (unknown_encodings, encoding))
+		 return;
+	g_hash_table_insert (unknown_encodings, g_strdup (encoding), GINT_TO_POINTER (1));
+	g_warning ("iconv does not support ppd character encoding: %s, trying CSISOLatin1",
+		   encoding);
+}
 
 static char *
 get_paper_text (ppd_file_t *ppd, ppd_size_t *size)
 {
 	/* This is dumb and slow and ugly and crappy and I hate myself. (Dave)*/
-	int i;
+	int i, j, k, len;
 	char *result;
-	static gboolean warn = FALSE;
+	ppd_group_t *group;
 
 	for (i = 0; i < ppd->num_groups; i++) {
-		ppd_group_t *group = &ppd->groups[i];
-		int j;
+		group = &ppd->groups[i];
 		for (j = 0; j < group->num_options; j++) {
 			ppd_option_t *option = &group->options[j];
-			if (!strcmp (option->keyword, "PageSize")) {
-				int k;
-				for (k = 0; k < option->num_choices; k++) {
-					if (!strcmp (option->choices[k].choice,
-						     size->name)) {
-						result = g_convert (option->choices[k].text, 
-								    strlen (option->choices[k].text), 
-								    "UTF8", 
-								    ppd->lang_encoding, 
-								    NULL, NULL, NULL);
-						if (result != NULL)
-							return result;
-						if (!warn) {
-							warn = TRUE;
-							g_warning ("iconv does not support ppd "
-								   "character encoding: %s, trying "
-								   "CSISOLatin1", 
-								   ppd->lang_encoding);
-						}
-						return g_convert (option->choices[k].text, 
-									  strlen (option->choices[k].text),
-									  "UTF8", 
-									  "CSISOLatin1", 
-									  NULL, NULL, NULL);
-					}
+			if (strcmp (option->keyword, "PageSize"))
+				continue;
+			for (k = 0; k < option->num_choices; k++) {
+				if (strcmp (option->choices[k].choice, size->name))
+					continue;
+				len = strlen (option->choices[k].text);
+				if (ppd->lang_encoding) {
+					result = g_convert (option->choices[k].text, len,
+							    "UTF8", ppd->lang_encoding,
+							    NULL, NULL, NULL);
+					if (result != NULL)
+						return result;
+					warn_of_unknown_encoding (ppd->lang_encoding);
 				}
+				return g_convert (option->choices[k].text,  len,
+						  "UTF8",	"CSISOLatin1",
+						  NULL, NULL, NULL);
 			}
 		}
 	}
-	
-	result = g_convert (size->name, 
-			    strlen (size->name), 
-			    "UTF8", 
-			    ppd->lang_encoding, 
+
+	len = strlen (size->name);
+	result = g_convert (size->name,  len, "UTF8", ppd->lang_encoding,
 			    NULL, NULL, NULL);
 	if (result != NULL)
 		return result;
-	if (!warn) {
-		warn = TRUE;
-		g_warning ("iconv does not support ppd character encoding:"
-			   " %s, trying CSISOLatin1", ppd->lang_encoding);
-	}
-	return g_convert (size->name, 
-			  strlen (size->name), 
-			  "UTF8", 
-			  "CSISOLatin1", 
+	warn_of_unknown_encoding (ppd->lang_encoding);
+	return g_convert (size->name,  len, "UTF8", "CSISOLatin1",
 			  NULL, NULL, NULL);
 }
 
@@ -232,10 +298,10 @@ option_list_new_with_default (GPANode      *parent,
 
 	choice = ppdFindChoice (option, defchoice);
 	g_free (defchoice);
-	
+
 	if (!choice && option->num_choices > 0)
 		choice = &option->choices[0];
-		
+
 	if (!choice)
 		return NULL;
 
@@ -256,12 +322,12 @@ load_paper_sizes (ppd_file_t *ppd, GPANode *parent)
 		 * or create a default one w/ A4 & USLetter? (Chema)
 		 */
 		return NULL;
-	
+
 	node = option_list_new_with_default (parent, "PhysicalSize",
 					     option);
 	if (!node)
 		return NULL;
-			
+
 	for (i = 0; i < ppd->num_sizes; i++) {
 		gchar *paper_name;
 
@@ -271,19 +337,19 @@ load_paper_sizes (ppd_file_t *ppd, GPANode *parent)
 			GPANode *size;
 			gchar *height;
 			gchar *width;
-			size = gpa_option_item_new (node, 
-						    ppd->sizes[i].name, 
+			size = gpa_option_item_new (node,
+						    ppd->sizes[i].name,
 						    paper_name);
 			g_free (paper_name);
-			
-			width  = g_strdup_printf ("%d", 
+
+			width  = g_strdup_printf ("%d",
 						  (int)ppd->sizes[i].width);
-			height = g_strdup_printf ("%d", 
+			height = g_strdup_printf ("%d",
 						  (int)ppd->sizes[i].length);
 
 			gpa_option_key_new (size, "Width",  width);
 			gpa_option_key_new (size, "Height", height);
-			
+
 			g_free (width);
 			g_free (height);
 		}
@@ -304,12 +370,12 @@ load_paper_sources (ppd_file_t *ppd, GPANode *parent)
 	option = ppdFindOption (ppd, "InputSlot");
 	if (!option)
 		return;
-	
+
 	node = option_list_new_with_default (parent, "PaperSource",
 					     option);
 	if (!node)
 		return;
-	
+
 	for (i = 0; i < option->num_choices; i++)
 		gpa_option_item_new (node,
 				     option->choices[i].choice,
@@ -325,12 +391,12 @@ static void
 load_cups_hold_types (GPANode *parent)
 {
 	GPANode *node;
-	
+
 	node = gpa_option_list_new (parent, "Hold", "no-hold");
 
 	if (!node)
 		return;
-	
+
 	gpa_option_item_new (node,"weekend",_("on the weekend"));
 	gpa_option_item_new (node,"third-shift",
 			     _("between midnight and 8 a.m."));
@@ -356,7 +422,7 @@ get_model (const gchar *printer, ppd_file_t *ppd)
 	id = g_strdup_printf ("Cups-%s-%s", ppd->manufacturer, ppd->nickname);
 
 	model = gpa_model_get_by_id (id, TRUE);
-	
+
 	if (model) {
 		g_free (id);
 		return GPA_MODEL (model);
@@ -379,7 +445,7 @@ get_model (const gchar *printer, ppd_file_t *ppd)
 	gpa_node_unref (job);
 
 	g_free (id);
-	
+
 	return (GPAModel *)model;
 }
 
@@ -392,7 +458,7 @@ get_model_no_ppd (const gchar *printer)
 	char *xml;
 
 	model = gpa_model_get_by_id ("Cups-unknown-unknown", TRUE);
-	
+
 	if (model)
 		return GPA_MODEL (model);
 
@@ -412,128 +478,70 @@ get_model_no_ppd (const gchar *printer)
 	return (GPAModel *)model;
 }
 
-static gboolean
-append_printer (GPAList *printers_list, GnomeCupsPrinter *cupsprinter,
-		gboolean is_default, const gchar *path)
-{
-	gboolean retval = FALSE;
-	const char *name = gnome_cups_printer_get_name (cupsprinter);
-	GPANode *printer  = gpa_printer_new_stub (name, name, path);
-
-	if (printer != NULL && gpa_node_verify (printer)) {
-		gpa_list_prepend (printers_list, printer);
-		if (is_default) {
-			gpa_list_set_default (printers_list, printer);
-			gpa_module_load_data (GPA_PRINTER (printer));
-		}
-		retval = TRUE;
-		g_signal_connect (cupsprinter, "gone",
-				  G_CALLBACK (printer_gone_cb),
-				  printers_list);
-	}
-
-	if (retval == FALSE) {
-		g_warning ("The CUPS printer %s could not be created\n", name);
-		my_gpa_node_unref (printer);
-	}
-
-	return retval;
-}
-
-static 	GModule *handle = NULL;
-
 static void
-gnome_print_cups_printer_list_append (gpointer printers_list, 
-				      const gchar *path)
-{
-	/* const gchar *def = cupsGetDefault(); 
-	   not reliable, dests[i].is_default used instead */
-        GList *printers = NULL;
-	GList *link;
-	struct GnomePrintCupsNewPrinterCbData *data;
-
-	g_return_if_fail (printers_list != NULL);
-	g_return_if_fail (GPA_IS_LIST (printers_list));
-
-	/* make ourselves resident */
-	if (!handle) 
-		handle = g_module_open (path, G_MODULE_BIND_LAZY);
-
-	gnome_cups_init (NULL);
-	data = g_new0 (struct GnomePrintCupsNewPrinterCbData, 1);
-	data->list = printers_list;
-	data->path = g_strdup (path);
-	gnome_cups_printer_new_printer_notify_add ((GnomeCupsPrinterAddedCallback) printer_added_cb, data);
-	
-	printers = gnome_cups_get_printers ();
-	
-	for (link = printers; link; link = link->next) {
-		const char *name;
-		GnomeCupsPrinter *printer;
-
-		name = link->data;
-		printer = gnome_cups_printer_get (name);
-
-		if (printer) {
-			append_printer (GPA_LIST (printers_list),
-					printer,
-					gnome_cups_printer_get_is_default (printer),					
-					path);
-			g_object_unref (G_OBJECT (printer));
-		}
-	}
-
-	gnome_cups_printer_list_free (printers);
-	
-	return;
-}
-
-static void
-gnome_print_cups_adjust_settings (GPASettings * settings,
+gnome_print_cups_adjust_settings (GPANode	   *settings,
 				  GnomeCupsPrinter *printer)
 {
-	char *value;
-	
-	value = gnome_cups_printer_get_option_value (printer, "PageSize");
-	if (value != NULL) {
-		gpa_node_set_path_value (GPA_NODE (settings),
-					 "Output.Media.PhysicalSize", 
-					 value);
-	}
-	g_free (value);
+	/* default to single sided */
+	gboolean duplex = FALSE, bind_short_edge = FALSE;
+	char *val;
 
-	value = gnome_cups_printer_get_option_value (printer, "PageSize");
-	if (value != NULL) {
-		if (strcmp("two-sided-long-edge", value) == 0) {
-			gpa_node_set_path_value (GPA_NODE (settings),
-						 "Output.Job.Duplex", 
-						 "true");
-			gpa_node_set_path_value (GPA_NODE (settings),
-						 "Output.Job.Tumble", 
-						 "false");
-		}
-		if (strcmp("two-sided-short-edge", value) == 0) {
-			gpa_node_set_path_value (GPA_NODE (settings),
-						 "Output.Job.Duplex", 
-						 "true");
-			gpa_node_set_path_value (GPA_NODE (settings),
-						 "Output.Job.Tumble", 
-						 "true");
-		}
-		if (strcmp("one-sided", value) == 0) {
-			gpa_node_set_path_value (GPA_NODE (settings),
-						 "Output.Job.Duplex", 
-						 "false");
-		}
+	val = gnome_cups_printer_get_option_value (printer, "PageSize");
+	if (val != NULL) {
+		gpa_node_set_path_value (settings,
+			"Output.Media.PhysicalSize", val);
+		g_free (val);
 	}
-	g_free (value);
-/* 	for (i=0; i < num_options; i++) { */
-/* 		g_print ("Option %s = %s\n", options[i].name, options[i].value); */
-/* 	} */
+
+	/* this set is pulled from cups/options.c */
+	val = gnome_cups_printer_get_option_value (printer, "Duplex");	/* std */
+	if (NULL == val)
+		val = gnome_cups_printer_get_option_value (printer, "JCLDuplex");	/* Samsung */
+	if (NULL == val)
+		val = gnome_cups_printer_get_option_value (printer, "EFDuplex");	/* EFI */
+	if (NULL == val)
+		val = gnome_cups_printer_get_option_value (printer, "KD03Duplex");	/* Kodak */
+	if (NULL != val) {
+		/* we do not really need to ignore case, but it does not hurt */
+		if (g_ascii_strcasecmp (val, "None") == 0) {
+			/* this is the default */
+		} else if (g_ascii_strcasecmp (val, "DuplexNoTumble") == 0) {
+			duplex = TRUE;
+			bind_short_edge = FALSE;
+		} else if (g_ascii_strcasecmp (val, "DuplexTumble") == 0) {
+			duplex = TRUE;
+			bind_short_edge = TRUE;
+		} else {
+			g_warning ("Unknown Duplex setting == '%s'", val);
+		}
+		g_free (val);
+	}
+	gpa_node_set_path_value (settings, "Output.Job.Duplex",
+		duplex ? "true" : "false");
+	gpa_node_set_path_value (settings, "Output.Job.Tumble",
+		(duplex && bind_short_edge) ? "true" : "false"); /* be anal */
 }
 
 static void
-attributes_changed_cb (GnomeCupsPrinter *cupsprinter,
+add_printer_location (GnomeCupsPrinter *cupsprinter,
+		      GPAPrinter       *printer)
+{
+	GPANode *state;
+	GPANode *location;
+	const char *str;
+
+	state = gpa_printer_get_state (printer);
+	location = gpa_node_get_child_from_path (state, "Location");
+	if (location == NULL) {
+		location = GPA_NODE (gpa_state_new ("Location"));
+		gpa_node_attach (state, location);
+	}
+	str = gnome_cups_printer_get_location (cupsprinter);
+	gpa_node_set_value (location, str);
+}
+
+static void
+cb_attributes_changed (GnomeCupsPrinter *cupsprinter,
 		       GPAPrinter     *printer)
 {
 	GPANode *state;
@@ -562,57 +570,121 @@ attributes_changed_cb (GnomeCupsPrinter *cupsprinter,
 }
 
 static void
-add_printer_location (GnomeCupsPrinter *cupsprinter,
-		      GPAPrinter       *printer)
+cb_get_ppd (guint id, ppd_file_t *ppd, GError **error, gpointer user_data)
 {
-	GPANode *state;
-	GPANode *location;
-	const char *str;
+	GnomePrintCupsLoadPrinterData *data = user_data;
+	const char *name;
+	GnomeCupsPrinter *cupsprinter;
+	GPANode *printer  = NULL;
+	GPANode *settings = NULL;
+	gboolean success = FALSE;
+	GPAModel *model = NULL;
+	GPAList *printers_list;
 
-	state = gpa_printer_get_state (printer);
-	location = gpa_node_get_child_from_path (state, "Location");
-	if (location == NULL) {
-		location = GPA_NODE (gpa_state_new ("Location"));
-		gpa_node_attach (state, location);
+	cupsprinter = data->printer;
+	printers_list = data->list;
+	name = gnome_cups_printer_get_name (data->printer);
+
+	if (ppd) {
+		model = get_model (name, ppd);
+	} else {
+		g_message ("The ppd file for the CUPS printer %s "
+			   "could not be loaded.", name);
+		model = get_model_no_ppd (name);
+	}
+
+	if (model == NULL) {
+		g_warning ("Couldn't create model for %s!\n", name);
+		goto out;
+	}
+
+	settings = gpa_settings_new (model, "Default", "SetIdFromCups");
+	if (settings == NULL) {
+		g_warning ("Couldn't create settings for %s!\n", name);
+		goto out;
+	}
+
+	printer = gpa_printer_new_stub (name, name, data->module_path);
+
+	if (printer != NULL && gpa_node_verify (printer)) {
+		gnome_print_cups_adjust_settings (settings, cupsprinter);
+
+		success = gpa_printer_complete_stub (GPA_PRINTER (printer),
+						     model,
+						     GPA_SETTINGS (settings));
+
+		if (success) {
+			gpa_list_prepend (printers_list, printer);
+			if (gnome_cups_printer_get_is_default (cupsprinter))
+				gpa_list_set_default (printers_list, printer);
+		}
+	} else {
+		g_warning ("The CUPS printer %s could not be created\n", name);
+		my_gpa_node_unref (printer);
+		goto out;
+	}
+
+	if (success) {
+		add_printer_location (cupsprinter, GPA_PRINTER (printer));
+		cb_attributes_changed (cupsprinter, GPA_PRINTER (printer));
+	}
+
+	/* Do we have to add any further instances */
+	/* FIXME - punt on instance bits for now */
+	/* 	{ */
+	/* 		char *id = g_strdup_printf ("SetIdFromCups-%s", printer->name); */
+	/* 		settings = gpa_settings_new (model, printer->name, id); */
+	/* 		g_free (id); */
+	/* 		id = NULL; */
+	/* 		if (settings != NULL) { */
+	/* 			gnome_print_cups_adjust_settings (GPA_SETTINGS (settings), */
+	/* 							  cupsprinter); */
+	/* 			gpa_list_prepend (GPA_NODE (printer->settings), */
+	/* 					  GPA_NODE (settings)); */
+	/* 			GPA_SETTINGS (settings)->printer */
+	/* 				= gpa_reference_new (GPA_NODE (printer), "Printer"); */
+	/* 		} */
+	/* 	} */
+
+out:
+	if (success == FALSE) {
+		g_warning ("The data for the CUPS printer %s "
+			   "could not be loaded.", name);
+
+		if (model != NULL)
+			my_gpa_node_unref (GPA_NODE (model));
+		if (settings != NULL)
+			my_gpa_node_unref (settings);
+	}
+
+	if (ppd) {
+		ppdClose (ppd);
 	}
-	str = gnome_cups_printer_get_location (cupsprinter);
-	gpa_node_set_value (location, str);
 }
 
 static void
-printer_added_cb (const char *name, struct GnomePrintCupsNewPrinterCbData *data)
+gnome_print_cups_printer_list_append (gpointer printers_list,
+				      const gchar *module_path)
 {
-	GPANode *node;
-	GnomeCupsPrinter *printer;
+	GnomePrintCupsNewPrinterCbData *data;
 
-	if ((node = gpa_printer_get_by_id (name)) != NULL) {
-		gpa_node_unref (node);
-		return;
-	}
+	g_return_if_fail (printers_list != NULL);
+	g_return_if_fail (GPA_IS_LIST (printers_list));
 
-	printer = gnome_cups_printer_get (name);
-	if (printer != NULL) {
-		append_printer (data->list, printer, FALSE, data->path);
-		g_object_unref (printer);
-	}
-	else
-		g_warning ("Printer %s does not exist!", name);
-	
-}
+	/* make ourselves resident */
+	if (!handle)
+		handle = g_module_open (module_path, G_MODULE_BIND_LAZY);
 
-static void
-printer_gone_cb (GnomeCupsPrinter *printer, GPAList *list)
-{
-	const char *name = gnome_cups_printer_get_name (printer);
-	GPANode *child;
-	child = gpa_node_get_child (GPA_NODE (list), NULL);
-	while (child) {
-		if (GPA_NODE_ID_COMPARE (child, name))
-			break;
-		child = gpa_node_get_child (GPA_NODE (list), child);
-	}
-	g_return_if_fail (child != NULL);
-	gpa_node_detach (child);
+	/* install cb_printer_added *before* calling gnome_cups_init(),
+	 * since that gets the list of printers */
+	data = g_new0 (GnomePrintCupsNewPrinterCbData, 1);
+	data->list = printers_list;
+	data->module_path = g_strdup (module_path);
+	gnome_cups_printer_new_printer_notify_add (
+		(GnomeCupsPrinterAddedCallback) cb_printer_added, data);
+	gnome_cups_printer_printer_removed_notify_add (
+		(GnomeCupsPrinterRemovedCallback) cb_printer_removed, printers_list);
+	gnome_cups_init (NULL);
 }
 
 static void
@@ -621,9 +693,9 @@ start_polling (GPAPrinter *printer)
 	GnomeCupsPrinter *cupsprinter;
 
 	cupsprinter = gnome_cups_printer_get (printer->name);
-	attributes_changed_cb (cupsprinter, printer);
-	g_signal_connect_object (cupsprinter, "attributes-changed", 
-				 G_CALLBACK (attributes_changed_cb), printer, 0);
+	cb_attributes_changed (cupsprinter, printer);
+	g_signal_connect_object (cupsprinter, "attributes-changed",
+				 G_CALLBACK (cb_attributes_changed), printer, 0);
 }
 
 static void
@@ -633,7 +705,7 @@ stop_polling (GPAPrinter *printer)
 
 	cupsprinter = gnome_cups_printer_get (printer->name);
 	g_signal_handlers_disconnect_by_func (cupsprinter,
-					      G_CALLBACK (attributes_changed_cb),
+					      G_CALLBACK (cb_attributes_changed),
 					      printer);
 	/* Unref twice since _get refs itself */
 	g_object_unref (G_OBJECT (cupsprinter));
@@ -655,79 +727,6 @@ G_MODULE_EXPORT void gpa_module_polling (GPAPrinter *printer,
 
 G_MODULE_EXPORT void gpa_module_load_data (GPAPrinter *printer)
 {
-	GPANode *settings = NULL;
-	GPAModel *model    = NULL;
-	const char *name   = printer->name;
-	ppd_file_t *ppd    = NULL;
-	gboolean success   = FALSE;
-	GnomeCupsPrinter *cupsprinter;
-	
-	if (printer->is_complete)
-		return;
-
-	cupsprinter = gnome_cups_printer_get (printer->name);
-
-	if (cupsprinter) {
-		ppd = gnome_cups_printer_get_ppd (cupsprinter);
-		if (ppd) {
-			model = get_model (name, ppd);
-		}
-	}
-	if (!ppd) {
-		g_warning ("The ppd file for the CUPS printer %s "
-			   "could not be loaded.", name);
-		model = get_model_no_ppd (name);
-	}
-	if (model == NULL)
-		goto gpa_module_load_data_exit;
-
-	settings = gpa_settings_new (model, "Default", "SetIdFromCups");
-	if (settings == NULL)
-		goto gpa_module_load_data_exit;
-
-	gnome_print_cups_adjust_settings (GPA_SETTINGS (settings), cupsprinter);
-
-	success = gpa_printer_complete_stub (printer, model, 
-					     GPA_SETTINGS (settings));
-
-	add_printer_location (cupsprinter, printer);
-	attributes_changed_cb (cupsprinter, printer);
-
-	/* Do we have to add any further instances */
-	/* FIXME - punt on instance bits for now */
-/* 	{ */
-/* 		char *id = g_strdup_printf ("SetIdFromCups-%s", name); */
-/* 		settings = gpa_settings_new (model, name, id); */
-/* 		g_free (id); */
-/* 		id = NULL; */
-/* 		if (settings != NULL) { */
-/* 			gnome_print_cups_adjust_settings (GPA_SETTINGS (settings), */
-/* 							  cupsprinter); */
-/* 			gpa_list_prepend (GPA_NODE (printer->settings), */
-/* 					  GPA_NODE (settings)); */
-/* 			GPA_SETTINGS (settings)->printer */
-/* 				= gpa_reference_new (GPA_NODE (printer), "Printer"); */
-/* 		} */
-/* 	} */
-
- gpa_module_load_data_exit:
-	g_object_unref (cupsprinter);
-	if (success == FALSE) {
-		g_warning ("The data for the CUPS printer %s "
-			   "could not be loaded.", name);
-
-		if (model != NULL)
-			my_gpa_node_unref (GPA_NODE (model));
-		if (settings != NULL)
-			my_gpa_node_unref (settings);
-	}
-
-	if (ppd) {
-		ppdClose (ppd);
-	}
-
-/* 	gpa_utils_dump_tree (GPA_NODE (printer),5); */
-	return;
 }
 
 
