@@ -26,6 +26,21 @@
 #include "sft.h"
 #include "gnome-print-tt-subset.h"
 
+static char *
+gnome_print_tmpnam (void)
+{
+	gchar *tmp;
+	gint fd;
+
+	/* see bug #164173 - we need to create these files in a proper tmp directory.
+	   we can't trust tmpnam(NULL) to behave properly. */
+	tmp = g_build_filename (g_get_tmp_dir (), "gnome-print-subset-XXXXXX", NULL);
+	fd = g_mkstemp (tmp);
+	close (fd);
+
+	return tmp;
+}
+
 void gnome_print_pdf_tt_create_subfont (const unsigned char *file_name, 
 			unsigned char **subfont_file, 
 			unsigned short *glyphArray, 
@@ -39,15 +54,16 @@ void gnome_print_pdf_tt_create_subfont (const unsigned char *file_name,
 	return;
     }
 
-    *subfont_file = tmpnam (NULL);
+    *subfont_file = gnome_print_tmpnam ();
 
-    CreateTTFromTTGlyphs(fnt, *subfont_file, glyphArray, encoding, len, 0, NULL, TTCF_AutoName | TTCF_IncludeOS2);
+    CreateTTFromTTGlyphs (fnt, *subfont_file, glyphArray, encoding, len, 0, NULL, TTCF_AutoName | TTCF_IncludeOS2);
 
     CloseTTFont(fnt);
 
 }
 
-void gnome_print_ps_tt_create_subfont (const unsigned char *file_name, 
+void
+gnome_print_ps_tt_create_subfont (const unsigned char *file_name, 
 			const unsigned char *encoded_font_name,
 			unsigned char **subfont_file, 
 			unsigned short *glyphArray, 
@@ -62,9 +78,9 @@ void gnome_print_ps_tt_create_subfont (const unsigned char *file_name,
 	return;
     }
 
-    *subfont_file = tmpnam (NULL);
-    subf = fopen (*subfont_file, "w");
-    CreateT42FromTTGlyphs (fnt, subf, encoded_font_name, (uint16 *)glyphArray, (byte *)encoding, len);
+    *subfont_file = gnome_print_tmpnam();
+    subf = fopen (*subfont_file, "wb");
+    CreateT42FromTTGlyphs (fnt, subf, encoded_font_name, (guint16 *)glyphArray, (guint8 *)encoding, len);
     fclose (subf);
 
     CloseTTFont(fnt);
